@@ -111,8 +111,7 @@ cm_casave_main_n(int fd, struct cm_store_ca *ca, struct cm_store_entry *e,
 					break;
 				default:
 					flags = NSS_INIT_READONLY |
-						NSS_INIT_NOROOTINIT |
-						NSS_INIT_NOMODDB;
+						NSS_INIT_NOROOTINIT;
 					/* Sigh.  Not a lot of detail.  Check
 					 * if we succeed in read-only mode,
 					 * which we'll interpret as lack of
@@ -163,7 +162,6 @@ cm_casave_main_n(int fd, struct cm_store_ca *ca, struct cm_store_entry *e,
 			decoded = CERT_DecodeCertFromPackage(package,
 							     strlen(package));
 			p = state->certs[i]->nickname;
-			ttrust = ",,";
 			switch (state->certs[i]->level) {
 			case root:
 			case other_root:
@@ -178,6 +176,9 @@ cm_casave_main_n(int fd, struct cm_store_ca *ca, struct cm_store_entry *e,
 					ttrust = ",,";
 				}
 				break;
+			default:
+				ttrust = ",,";
+				break;
 			}
 			memset(&trust, 0, sizeof(trust));
 			CERT_DecodeTrustString(&trust, ttrust);
