@@ -306,8 +306,7 @@ main(int argc, const char **argv)
 			       NULL, NULL, NULL, NULL,
 			       NSS_INIT_NOCERTDB |
 			       NSS_INIT_READONLY |
-			       NSS_INIT_NOROOTINIT |
-			       NSS_INIT_NOMODDB);
+			       NSS_INIT_NOROOTINIT);
 	if (nctx == NULL) {
 		cm_log(1, "Unable to initialize NSS.\n");
 		_exit(1);
@@ -536,6 +535,7 @@ main(int argc, const char **argv)
 					       CM_SUBMIT_CSR_ENV);
 				}
 				poptPrintUsage(pctx, stdout, 0);
+				free(csr);
 				return CM_SUBMIT_STATUS_UNCONFIGURED;
 			}
 			csr = cm_submit_u_url_encode(csr);
@@ -588,6 +588,8 @@ main(int argc, const char **argv)
 			params = talloc_asprintf(ctx,
 						 "%s&%s=%s",
 						 params, p, q);
+			free(p);
+			free(q);
 		}
 		use_agent_approval = FALSE;
 		break;
