@@ -226,6 +226,11 @@ cm_keygen_n_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 			break;
 		}
 	}
+	NSS_ShutdownContext(ctx);
+	ctx = NSS_InitContext(entry->cm_key_storage_location,
+			      NULL, NULL, NULL, NULL,
+			      (readwrite ? 0 : NSS_INIT_READONLY) |
+			      NSS_INIT_NOROOTINIT);
 	reason = util_n_fips_hook();
 	if (reason != NULL) {
 		cm_log(1, "Error putting NSS into FIPS mode: %s\n", reason);
@@ -272,6 +277,9 @@ cm_keygen_n_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 		cm_log(1, "Error locating token for key generation.\n");
 		_exit(CM_SUB_STATUS_ERROR_NO_TOKEN);
 	}
+	if (entry->cm_cert_token == NULL) {
+		entry->cm_cert_token = talloc_strdup(entry, util_internal_token_name());
+	}
 	/* Walk the list looking for the requested slot, or the first one if
 	 * none was requested. */
 	slot = NULL;
@@ -400,8 +408,8 @@ next_slot:
 	    (strlen(pin) > 0) &&
 	    (cb_data.n_attempts == 0)) {
 		cm_log(1, "PIN was not needed to auth to key "
-		       "store, though one was provided. "
-		       "Treating this as an error.\n");
+		       "store token %s, though one was provided. "
+		       "Treating this as an error.\n", token);
 		PK11_FreeSlotList(slotlist);
 		error = NSS_ShutdownContext(ctx);
 		if (error != SECSuccess) {
@@ -588,7 +596,6 @@ retry_gen:
 			break;
 		}
 	}
-	generated_size = SECKEY_PublicKeyStrengthInBits(pubkey);
 	cm_log(1, "Ended up with %d bit public key.\n",
 	       SECKEY_PublicKeyStrengthInBits(pubkey));
 	/* Check for keys with the desired name, selecting a new name if
