@@ -115,6 +115,11 @@ cm_keyiread_n_get_keys(struct cm_store_entry *entry, int readwrite)
 			break;
 		}
 	}
+	NSS_ShutdownContext(ctx);
+	ctx = NSS_InitContext(entry->cm_key_storage_location,
+			      NULL, NULL, NULL, NULL,
+			      (readwrite ? 0 : NSS_INIT_READONLY) |
+			      NSS_INIT_NOROOTINIT);
 	reason = util_n_fips_hook();
 	if (reason != NULL) {
 		cm_log(1, "Error putting NSS into FIPS mode: %s\n", reason);
@@ -152,6 +157,9 @@ cm_keyiread_n_get_keys(struct cm_store_entry *entry, int readwrite)
 		_exit(CM_SUB_STATUS_ERROR_AUTH);
 	}
 	PK11_SetPasswordFunc(&cm_pin_read_for_cert_nss_cb);
+	if (entry->cm_key_token == NULL) {
+		entry->cm_key_token = talloc_strdup(entry, util_internal_token_name());
+	}
 	n_tokens = 0;
 	pubkey = NULL;
 	/* In practice, the internal slot is either a non-storage slot (in
@@ -337,8 +345,12 @@ cm_keyiread_n_get_keys(struct cm_store_entry *entry, int readwrite)
 			     cnode = CERT_LIST_NEXT(cnode)) {
 				nickname = entry->cm_key_nickname;
 				cert = cnode->cert;
+				es = talloc_asprintf(entry, "%s:%s",
+									         entry->cm_cert_token,
+											 entry->cm_cert_nickname);
 				if ((nickname != NULL) &&
-				    (strcmp(cert->nickname, nickname) == 0)) {
+				    ((strcmp(cert->nickname, nickname) == 0) ||
+					(strcmp(cert->nickname, es) == 0))) {
 					cm_log(3, "Located a certificate with "
 					       "the key's nickname (\"%s\").\n",
 					       nickname);
@@ -489,7 +501,6 @@ cm_keyiread_n_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 	readwrite = settings->readwrite;
 	keys = cm_keyiread_n_get_keys(entry, readwrite);
 	alg = "";
-	size = 0;
 	if (keys != NULL) {
 		switch (SECKEY_GetPrivateKeyType(keys->privkey)) {
 		case rsaKey:
