@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010,2011,2012,2014 Red Hat, Inc.
+ * Copyright (C) 2010,2011,2012,2014,2015 Red Hat, Inc.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -107,6 +107,14 @@ cm_prefs_preferred_cipher(void)
 			free(cipher);
 			return cm_prefs_aes256;
 		}
+		if (strcasecmp(cipher, "des") == 0) {
+			free(cipher);
+			return cm_prefs_des;
+		}
+		if (strcasecmp(cipher, "des3") == 0) {
+			free(cipher);
+			return cm_prefs_des3;
+		}
 		free(cipher);
 	}
 	return cm_prefs_aes128;
@@ -118,6 +126,11 @@ cm_prefs_preferred_digest(void)
 	char *digest;
 	digest = cm_prefs_config(NULL, "digest");
 	if (digest != NULL) {
+		if ((strcasecmp(digest, "md5") == 0) ||
+		    (strcasecmp(digest, "md-5") == 0)) {
+			free(digest);
+			return cm_prefs_md5;
+		}
 		if ((strcasecmp(digest, "sha1") == 0) ||
 		    (strcasecmp(digest, "sha-1") == 0)) {
 			free(digest);
@@ -267,7 +280,7 @@ cm_prefs_default_ca(void)
 }
 
 const char *
-cm_prefs_validity_period(void)
+cm_prefs_selfsign_validity_period(void)
 {
 	static const char *period;
 	if (period == NULL) {
@@ -279,6 +292,23 @@ cm_prefs_validity_period(void)
 	return period;
 }
 
+const char *
+cm_prefs_local_validity_period(void)
+{
+	static const char *period;
+
+	if (period == NULL) {
+		period = cm_prefs_config("local", "validity_period");
+		if (period == NULL) {
+			period = cm_prefs_config("selfsign", "validity_period");
+			if (period == NULL) {
+				period = CM_DEFAULT_CERT_LIFETIME;
+			}
+		}
+	}
+	return period;
+}
+
 static const char *
 yes_words[] = {"yes", "y", "true", "t", "1"};
 
@@ -489,3 +519,62 @@ cm_prefs_preferred_key_algorithm(void)
 	}
 	return CM_DEFAULT_PUBKEY_TYPE;
 }
+
+const char *
+cm_prefs_nss_ca_trust(void)
+{
+	static const char *trust;
+
+	if (trust == NULL) {
+		trust = cm_prefs_config(NULL, "nss_ca_trust");
+	}
+	return trust;
+}
+
+const char *
+cm_prefs_nss_other_trust(void)
+{
+	static const char *trust;
+
+	if (trust == NULL) {
+		trust = cm_prefs_config(NULL, "nss_other_trust");
+	}
+	return trust;
+}
+
+long long
+prefs_key_end_of_life(time_t ref)
+{
+	const char *cfg;
+	time_t tmp;
+
+	tmp = -1;
+	cfg = cm_prefs_config(NULL, "max_key_lifetime");
+	if (cfg != NULL) {
+		if (cm_submit_u_delta_from_string(cfg, ref, &tmp) == 0) {
+			return tmp;
+		}
+	}
+	return -1;
+}
+
+long
+prefs_max_key_use_count(void)
+{
+	static long count = -2;
+	long tmp;
+	const char *cfg;
+	char *p;
+
+	if (count == -2) {
+		count = -1;
+		cfg = cm_prefs_config(NULL, "max_key_use_count");
+		if (cfg != NULL) {
+			tmp = strtol(cfg, &p, 10);
+			if ((p != NULL) && (*p == '\0')) {
+				count = tmp;
+			}
+		}
+	}
+	return count;
+}
