@@ -68,7 +68,6 @@
 
 #define OP_GET_CA_CAPS "GetCACaps"
 #define OP_GET_CA_CERT "GetCACert"
-#define OP_GET_CA_CERT_CHAIN "GetCACertChain"
 #define OP_GET_INITIAL_CERT "PKIOperation"
 #define OP_PKCSREQ "PKIOperation"
 enum known_ops {
@@ -339,6 +338,7 @@ main(int argc, const char **argv)
 	}
 	if (c != -1) {
 		poptPrintUsage(pctx, stdout, 0);
+		free(cainfo);
 		return CM_SUBMIT_STATUS_UNCONFIGURED;
 	}
 
@@ -368,11 +368,11 @@ main(int argc, const char **argv)
 		break;
 	case op_get_ca_caps:
 		/* Only step: read capabilities for the daemon. */
-		params = talloc_asprintf(ctx, "operation=" OP_GET_CA_CAPS "&message=%s", id);
+		params = talloc_asprintf(ctx, "operation=" OP_GET_CA_CAPS);
 		break;
 	case op_get_ca_certs:
 		/* First step: get the root certificate. */
-		params = talloc_asprintf(ctx, "operation=" OP_GET_CA_CERT "&message=%s", id);
+		params = talloc_asprintf(ctx, "operation=" OP_GET_CA_CERT);
 		break;
 	case op_get_cert_initial:
 		if ((racert == NULL) || (strlen(racert) == 0)) {
@@ -387,10 +387,11 @@ main(int argc, const char **argv)
 			}
 			if ((message == NULL) || (strlen(message) == 0)) {
 				printf(_("Error reading request.  Expected PKCS7 data containing a GetInitialCert pkiMessage, got nothing.\n"));
+				free(cainfo);
 				return CM_SUBMIT_STATUS_NEED_SCEP_MESSAGES;
 			}
 			/* First step: read capabilities for our use. */
-			params = talloc_asprintf(ctx, "operation=" OP_GET_CA_CAPS "&message=%s", id);
+			params = talloc_asprintf(ctx, "operation=" OP_GET_CA_CAPS);
 		}
 		break;
 	case op_pkcsreq:
@@ -406,10 +407,11 @@ main(int argc, const char **argv)
 			}
 			if ((message == NULL) || (strlen(message) == 0)) {
 				printf(_("Error reading request.  Expected PKCS7 data containing a PKCSReq pkiMessage, got nothing.\n"));
+				free(cainfo);
 				return CM_SUBMIT_STATUS_NEED_SCEP_MESSAGES;
 			}
 			/* First step: read capabilities for our use. */
-			params = talloc_asprintf(ctx, "operation=" OP_GET_CA_CAPS "&message=%s", id);
+			params = talloc_asprintf(ctx, "operation=" OP_GET_CA_CAPS);
 		}
 		break;
 	}
@@ -417,6 +419,7 @@ main(int argc, const char **argv)
 	/* Supply help output, if it's needed. */
 	if (missing_args) {
 		poptPrintUsage(pctx, stdout, 0);
+		free(cainfo);
 		return CM_SUBMIT_STATUS_UNCONFIGURED;
 	}
 
@@ -493,6 +496,7 @@ main(int argc, const char **argv)
 				verbose > 1 ?
 				cm_submit_h_curl_verbose_on :
 				cm_submit_h_curl_verbose_off);
+	free(cainfo);
 	cm_submit_h_run(hctx);
 	content_type = cm_submit_h_result_type(hctx);
 	if (content_type == NULL) {
@@ -523,13 +527,10 @@ main(int argc, const char **argv)
 		abort(); /* never reached */
 		break;
 	case op_get_ca_caps:
+	case op_get_ca_certs:
 		/* nothing to do here */
 		params2 = NULL;
 		break;
-	case op_get_ca_certs:
-		/* Step two: request the chain. */
-		params2 = talloc_asprintf(ctx, "operation=" OP_GET_CA_CERT_CHAIN "&message=%s", id);
-		break;
 	case op_get_cert_initial:
 		/* Step two: actually poll.  If we have multiple messages which
 		 * we can use, decide which one to use. */
@@ -792,8 +793,8 @@ main(int argc, const char **argv)
 				fprintf(stderr, "code_text = \"%s\"\n", cm_submit_h_result_code_text(hctx));
 				syslog(LOG_DEBUG, "%s %s?%s\n", "GET", url, params2);
 			}
-			if (strcasecmp(content_type2,
-				       "application/x-x509-ca-cert") != 0) {
+			if ((content_type2 != NULL) && (strcasecmp(content_type2,
+				       "application/x-x509-ca-cert") != 0)) {
 				if (verbose > 0) {
 					fprintf(stderr, "Content is not "
 						"\"application/x-x509-ca-cert\""
@@ -881,8 +882,8 @@ main(int argc, const char **argv)
 		break;
 	case op_get_cert_initial:
 	case op_pkcsreq:
-		if (strcasecmp(content_type2,
-			       "application/x-pki-message") == 0) {
+		if ((content_type2 != NULL) && (strcasecmp(content_type2,
+			       "application/x-pki-message") == 0)) {
 			memset(&cacerts, 0, sizeof(cacerts));
 			cacerts[0] = cacert ? cacert : racert;
 			cacerts[1] = cacert ? racert : NULL;
