@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010 Red Hat, Inc.
+ * Copyright (C) 2010,2015 Red Hat, Inc.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -56,9 +56,10 @@ struct cm_submit_h_context *cm_submit_h_init(void *parent,
 					     enum cm_submit_h_opt_env_modify env,
 					     enum cm_submit_h_opt_curl_verbose verbose);
 void cm_submit_h_run(struct cm_submit_h_context *ctx);
+int cm_submit_h_response_code(struct cm_submit_h_context *ctx);
 int cm_submit_h_result_code(struct cm_submit_h_context *ctx);
 const char *cm_submit_h_result_code_text(struct cm_submit_h_context *ctx);
-const char *cm_submit_h_results(struct cm_submit_h_context *ctx);
+const char *cm_submit_h_results(struct cm_submit_h_context *ctx, int *length);
 const char *cm_submit_h_result_type(struct cm_submit_h_context *ctx);
 
 #endif
