@@ -281,7 +281,7 @@ cm_submit_n_decrypt_envelope(const unsigned char *envelope,
 	PLArenaPool *arena = NULL;
 	SECStatus error;
 	NSSInitContext *ctx = NULL;
-	PK11SlotInfo *slot;
+	PK11SlotInfo *slot = NULL;
 	PK11SlotList *slotlist = NULL;
 	PK11SlotListElement *sle;
 	SECKEYPrivateKeyList *keylist = NULL;
@@ -317,6 +317,11 @@ cm_submit_n_decrypt_envelope(const unsigned char *envelope,
 		}
 		goto done;
 	}
+	NSS_ShutdownContext(ctx);
+	ctx = NSS_InitContext(args->entry->cm_key_storage_location,
+			      NULL, NULL, NULL, NULL,
+			      NSS_INIT_READONLY |
+			      NSS_INIT_NOROOTINIT);
 	reason = util_n_fips_hook();
 	if (reason != NULL) {
 		cm_log(1, "Error putting NSS into FIPS mode: %s\n", reason);
@@ -346,6 +351,9 @@ cm_submit_n_decrypt_envelope(const unsigned char *envelope,
 		cm_log(1, "Error reading PIN for key storage.\n");
 		goto done;
 	}
+	if (args->entry->cm_key_token == NULL) {
+		args->entry->cm_key_token = talloc_strdup(args->entry, util_internal_token_name());
+	}
 	PK11_SetPasswordFunc(&cm_pin_read_for_cert_nss_cb);
 	n_tokens = 0;
 	/* In practice, the internal slot is either a non-storage slot (in
@@ -402,7 +410,7 @@ cm_submit_n_decrypt_envelope(const unsigned char *envelope,
 		}
 		error = PK11_Authenticate(slot, PR_TRUE, &cb_data);
 		if (error != SECSuccess) {
-			cm_log(1, "Error authenticating to token "
+			cm_log(1, "submit-n: Error authenticating to token "
 			       "\"%s\".\n", token);
 			goto done;
 		}
