@@ -5,7 +5,7 @@ cd "$tmpdir"
 source "$srcdir"/functions
 initnssdb "$tmpdir"
 
-for size in 1024 1536 2048 3072 4096 ; do
+for size in 2048 3072 4096 ; do
 	# Build a self-signed certificate.
 	run_certutil -d "$tmpdir" -S -g $size -n keyi$size \
 		-s "cn=T$size" -c "cn=T$size" \
@@ -199,7 +199,7 @@ iterate() {
 
 iteration=1
 
-for size in 1024 ; do
+for size in 2048 ; do
 	iterate "$size" "$subject" "$hostname" "$email" "$principal" "$ku" "$eku" "$challengepassword" "$certfname" "$ca" "$capathlen" "$crldp" "$ocsp" "$nscomment" "$subjectder" "$ipaddress" "$freshestcrl" "$no_ocsp_check" "$profile" "$ns_certtype"
 done
 
@@ -299,7 +299,7 @@ done
 ns_certtype=
 
 size=2048
-subject="CN=Babs Jensen"
+subject="CN=Cloud\, Inc."
 hostname=localhost,localhost.localdomain
 email=root@localhost,root@localhost.localdomain
 principal=root@EXAMPLE.COM,root@FOO.EXAMPLE.COM
